import java.awt.*;
import java.awt.event.*;

/**
 * OrderForm.java
 *
 * The class which draws all the forms and does the functionality
 * for the applet. It extends from Panel class and implements 
 * the listener interfaces ActionListener, ItemListener and 
 * TextListener. It makes use of several components to build the
 * user interface for the applet and uses the AWT events to add
 * functionality for the components.
 * 
 * @author xjqiu@npac.syr.edu
 */

public class OrderForm extends Panel  // class definition
	implements ActionListener,ItemListener,TextListener{

 GridBagLayout gb = new GridBagLayout(); // layout for this component
 GridBagConstraints gbc = new GridBagConstraints(); // constraint class for the layout

// Textfield components for getting order number, 
// phone number, address etc. Their labels are also
// declared and defined here.

 private Label olabel = new Label("Name");
 private TextField name;
 private Label phlabel = new Label("Telephone Number");
 private TextField ph;
 private Label stlabel = new Label("Street/Apartment");
 private TextField st;
 private Label citylabel = new Label("City");
 private TextField city;

 // CheckboxGroup and Checkbox declarations for the
 // pay by radio buttons

 private Label paybylabel = new Label("Pay By");
 private CheckboxGroup payby = new CheckboxGroup();
 private Checkbox credit; 
 private Checkbox cash; 

 // CheckboxGroup and Checkbox components for the type
 // of pizza and their labels

 private Label pizlabel = new Label("Order Pizza");
 private CheckboxGroup pizgrp = new CheckboxGroup();
 private Checkbox none; 
 private Checkbox meat; 
 private String meatLabel = "Meat Lover's";
 private Checkbox veggie; 
 private String veggieLabel = "Veggie"; 
 private Checkbox cheese; 
 private String cheeseLabel = "Cheese";
 private Checkbox own; 
 private String ownLabel = "Make Your Own";

 // The two list components for
 // listing the ingredients of the pizza

 private IngredientList ilist;
 private IngredientList finalilist;

 // Buttons for adding and removing pizza
 // ingredients from the list

 private Button addi;
 private String addLabel = " -> ";
 private Button removei;
 private String remLabel = " <- ";

 // Buttons for printing and Canceling the order

 private Button conn;
 private String connLabel = "  Connection  ";
 private Button clear;
 private String clearLabel = "  Clear  ";
 private Button close;
 private String closeLabel = "  Close  ";

 
 private Color c = new Color(98,90,145); // color for the buttons

 // List of ingredients. First all the available ingredients and then
 // for a particular kind of pizza

 private String[] ingreds = { "Beef", "Cheese", "Chicken", "Mushrooms",
								"Olive","Onion", "Pepper","Pepperoni",
								"Sausage" };
 private String[] meatingreds = { "Beef", "Pepperoni", "Sausage" };
 private String[] veggieingreds = { "Mushrooms","Olive","Onion", "Pepper"};
 private String[] cheeseingreds = { "Cheese"};
 private DisplayPanel dpof;


/** 
 * Constructor for the class. This adds all the 
 * components at respective places in the Panel. 
 * It uses the Insets class to position the components
 * in the window.
 */

 public OrderForm(DisplayPanel dp) {
     dpof = dp;

     setLayout(gb); // set the layout as GridBagLayout
     setBackground(Color.white); // set the background color as white

     gbc.fill = GridBagConstraints.NONE;
     gbc.anchor = GridBagConstraints.NORTHWEST; // sets anchor as NORTHWEST
     setFont(new Font("Helvetica", Font.BOLD, 12)); // sets font OrderForm.java

     addComp(olabel,30,10,1,1,new Insets(95,25,0,1)); // adds the Label
     name = new TextField(15);
     name.addTextListener(this); // add a TextListener for this textfield
     addComp(name,40,10,1,1,new Insets(-97,0,0,1)); // adds the component

     addComp(phlabel,30,20,1,1,new Insets(-60,25,0,1));
     ph = new TextField(7);
     ph.addTextListener(this);  // places a phone text field
     addComp(ph,40,20,1,1,new Insets(-62,0,0,1));

     addComp(stlabel,30,30,1,1,new Insets(-25,25,0,1));
     st = new TextField(15); //places a state textfield
     st.addTextListener(this);
     addComp(st,40,30,1,1,new Insets(-27,0,0,1));

     addComp(citylabel,30,40,1,1,new Insets(5,25,0,1));
     city = new TextField(15); // places a city text field
     city.addTextListener(this);
     addComp(city,40,40,1,1,new Insets(3,0,0,1));
 
     addComp(paybylabel,30,60,1,1, new Insets(5,25,0,1));
     cash = new Checkbox("Cash", payby, true); // sets the payby option
     addComp(cash,40,60,1,3,new Insets(4,1,0,0));
     credit = new Checkbox("Credit Card",payby,false);
     addComp(credit,41,60,1,1,new Insets(4,0,0,1));

     // declares the checkboxgroup for the type of pizza 
     // and adds the pizza types to this group.
     // Also adds the ItemListener to all these 
     // Checkbox components.

     addComp(pizlabel,30,80,1,1, new Insets(7,25,0,1));
     none = new Checkbox("None",pizgrp,false);
     meat = new Checkbox(meatLabel,pizgrp,false);
     meat.addItemListener(this);
     addComp(meat,30,105,1,1,new Insets(4,45,0,1));
     veggie = new Checkbox(veggieLabel,pizgrp,false);
     veggie.addItemListener(this);
     addComp(veggie,30,130,1,1,new Insets(4,45,0,1));
     cheese = new Checkbox(cheeseLabel,pizgrp,false);
     cheese.addItemListener(this);
     addComp(cheese,30,155,1,1,new Insets(4,45,0,1));
     own = new Checkbox(ownLabel,pizgrp,false);
     own.addItemListener(this);
     addComp(own,30,200,1,1,new Insets(4,45,0,1));

     // adds the Ingredient List component

     ilist = new IngredientList(4,ingreds);
     addComp(ilist,30,210,1,1,new Insets(4,25,0,1));

     // Add buttons for adding and removing elements 
     // from the two Ingredient lists. Adds ActionListener 
     // to this component. 

     addi = new Button(addLabel);
     addi.setFont(new Font("Helvetica", Font.BOLD, 18));
     addi.addActionListener(this);
     addi.setEnabled(false);
     addComp(addi,32,210,1,1,new Insets(4,20,0,1));
     removei = new Button(remLabel);
     removei.setFont(new Font("Helvetica", Font.BOLD, 18));
     removei.addActionListener(this);
     removei.setEnabled(false);
     addComp(removei,32,210,1,1,new Insets(38,20,0,1));

     // adds another Ingredient List component

     finalilist = new IngredientList(5);
     addComp(finalilist,40,210,1,1,new Insets(4,25,0,1));

     // adds the print and Cancel buttons.


     gbc.anchor = GridBagConstraints.SOUTH;
     conn = new Button(connLabel);
     conn.setBackground(c);
     conn.setForeground(Color.white);
     conn.setFont(new Font("Helvetica", Font.PLAIN, 18));
     conn.addActionListener(this);
     addComp(conn,30,300,1,1,new Insets(0,160,0,-50));

	 clear = new Button(clearLabel);
     clear.setBackground(c);
     clear.setForeground(Color.white);
     clear.setFont(new Font("Helvetica", Font.PLAIN, 18));
     clear.addActionListener(this);
     addComp(clear,40,300,1,1,new Insets(0,0,0,0));

     close = new Button(closeLabel);
     close.setBackground(c);
     close.setForeground(Color.white);
     close.setFont(new Font("Helvetica", Font.PLAIN, 18));
     close.addActionListener(this);
     addComp(close,50,300,1,1,new Insets(0,-80,0,160));
 }

/**
 * Method that actually sets the parameters and adds the component
 * to the panel. Gets the coordinates, width and height and insets 
 * as well as the component object as arguments.
 */

 private void addComp
            ( Component c, int x, int y, int ht, int wt, Insets insets ) {
     gbc.gridx = x;
     gbc.gridy = y;
 
     gbc.gridwidth = wt;
     gbc.gridheight = ht;
  
     gbc.insets = insets;
     
     gb.setConstraints(c,gbc);
     add(c);
 }

/**
 * paint method for drawing a line below the components
 */
 public void paint(Graphics g) {
     g.drawLine(0,480,600,480);
 }

 //############# Functionality of the various components added ###########
 //############# above are given below   					   ###########

/**
 * ActionListener implementation. This method is called for 
 * all the button clicks. The getSource method is used to 
 * determine the origin of the event. Depending on the button
 * that was clicked the respective action is taken by calling 
 * one of those methods in this class.
 */ 

 public void actionPerformed(ActionEvent e) {
  
     Button b = (Button) e.getSource(); // Get the button object

     String tmpstr = b.getLabel(); // Get the label of the button

     if(tmpstr.equals(addLabel)) 
         addToList(); // call method to add to the list
     else if(tmpstr.equals(remLabel)) 
         remFromList(); // call the method to remove from the list
     else if(tmpstr.equals(connLabel)) 
         dpof.showResult(getOrderName()); // call method to print the values
     else if(tmpstr.equals(clearLabel))
         clearValues(); // call the method to clear all the components
     else
	         System.out.println("Not implemented yet");
     b = null;
     tmpstr = null; // why wait for garbage collection
 }
 
/**
 * ItemListener implementation. This method is called by the checkbox
 * components in the applet. It also uses the getSource method to
 * determine which checkbox was clicked and calls the appropriate
 * function handler.
 */

 public void itemStateChanged(ItemEvent e) {

     Checkbox cb = (Checkbox) e.getSource(); // get the Checkbox 
 
     String tmpstr = cb.getLabel(); // get the label of the Checkbox
 
     if(tmpstr.equals(meatLabel)){
         enableAddRem(); // enables the buttons
         meatToList(); // order for a meat pizza
     }else if(tmpstr.equals(veggieLabel)){
         enableAddRem(); // enables the buttons
         veggieToList(); // order for a veggie pizza
     }else if(tmpstr.equals(cheeseLabel)){ 
         enableAddRem(); // enables the buttons
         cheeseToList(); // order for a cheese pizza
     }else if(tmpstr.equals(ownLabel)){ 
         enableAddRem(); // enables the buttons
         ownToList(); // order for a make your own pizza
     }else 
         System.out.println("Unknown checkbox");
 
     cb = null;
     tmpstr = null; // Garbage collect immediately
 }

/**
 * TextListener Implementation. Any text value changes in the textfield
 * component will call this method. It uses the getSource method for 
 * getting which textfield generated this event and handles appropriately.
 * Mainly used for error checking purposes here. Checks for maximum
 * allowable lengths of the fields and truncates if necessary
 */

 public void textValueChanged(TextEvent e) {
     
     TextField tf = (TextField) e.getSource(); // get the Textfield object
     int len = tf.getColumns() + 4; //set the maxlength as size+4
     String tfstr = tf.getText(); // get the text from the textfield
     
    
     // Checks if the size is greater than allowable length. If yes
     // then prints an error message and chops the text to maxlength
     if((tfstr.length()) > len) {
         printResult(" Max length of the field is "+
                       Integer.toString(len));
         tf.setText(tfstr.substring(0,len));
     }
   
 }

/**
 * This methods gets the selected items from the ingredient list and 
 * adds them to the final ingredient list. Called by the Add button,
 * it removes the items that have been added to the final list.
 */

 private void addToList() {

     String[] str = ilist.getSelectedItems();
     for(int i=0; i < (str.length); i++ ) {
         finalilist.addToList(str[i]);    
         ilist.remove(str[i]);
     }
     str = null;
 }

/**
 * This methods gets the selected items from the final ingredient list 
 * and removes them to the initial ingredient list. Called by the Remove
 * button.
 */

 private void remFromList() {

     String[] str = finalilist.getSelectedItems();
     for(int i=0; i < (str.length); i++ ) {
         ilist.addToList(str[i]);    
         finalilist.remove(str[i]);
     }
     str = null;
 }

/**
 * Once everything is set, this method is called on clicking the
 * Print button at the bottom of the applet. It first checks for 
 * the values and if everything is alright it prints.
 */

 private void printValues() {
     if(checkValues()){ //checks if values are present
         printClear(); // clear the drawing area
//*******          getValues(); // get the values and prints
     }
 }

/**
 * Sets the defaults for the various components. This includes clearing 
 * the textfields and the drawing area  for the output
 */

 private void clearValues() {
     name.setText("");
     st.setText("");
     ph.setText("");
     city.setText("");
     payby.setSelectedCheckbox(cash);
     pizgrp.setSelectedCheckbox(none);
     ilist.removeAll();
     finalilist.removeAll();
     ilist.addToList(ingreds);
     addi.setEnabled(false);
     removei.setEnabled(false);
     printClear();
 }

/**
 * Enables the add and remove buttons 
 */
 private void enableAddRem() {
     addi.setEnabled(true);
     removei.setEnabled(true);
 }

/**
 * The meat pizza has been selected. This method puts the ingredients of 
 * this pizza to the final ingredients list.
 */

 private void meatToList() {
     finalilist.removeAll();
     finalilist.addToList(meatingreds);
     ilist.removeAll(); // clears the initial list
 }

/**
 * The veggie pizza has been selected. This method puts the ingredients of 
 * this pizza to the final ingredients list.
 */

 private void veggieToList() {
     finalilist.removeAll();
     finalilist.addToList(veggieingreds);
     ilist.removeAll(); // clears the initial list
 }

/**
 * The cheese pizza has been selected. This method puts the ingredients of 
 * this pizza to the final ingredients list.
 */
 private void cheeseToList() {
     finalilist.removeAll();
     finalilist.addToList(cheeseingreds);
     ilist.removeAll(); //clears the initial list
 }

/**
 * The 'make our own' pizza option has been selected. This method
 *  puts the ingredients of  this pizza to the final ingredients list.
 */

 private void ownToList() {
     ilist.removeAll();
     finalilist.removeAll();
     ilist.addToList(ingreds); // adds the full list to the final list
 }
 
/**
 * Checks if the values are present or else prints
 * error messages
 */

 private boolean checkValues() {
 
     Checkbox ch;

     if((name.getText().trim()).length() <= 0){
         printResult("Error ! No name present");
         return false;
     }else if((ph.getText().trim()).length() <= 0){
         printResult("Error ! No phone number present ");
         return  false;
     }else if((st.getText().trim()).length() <= 0){
         printResult("error ! No street address present ");
         return false;
     }else if((( ch = pizgrp.getSelectedCheckbox()) == null) ||
                ((ch.getLabel()).equals("None")) ){
         printResult("Error ! No Pizza selected");
         return false;
     }else if(finalilist.getItemCount() <= 0 ) {
         printResult("Error ! No ingredients present");
         return false;
     }
     return true;
 }

/**
 * Clears the drawing area for the output
 */

 private void printClear() {
//*******     Graphics g = getGraphics();
//*******     g.clearRect(30,450,470,20);
//*******     g.drawString(" " ,30,450);
 }

/**
 * Draws a single line output on the panel using the coordinates
 */

 private void printResult(String str) {
//     printClear();
     printResult(str,40,465);
 }

/**
 * Draws a string in the screen using the given coordinates
 */
 
 private void printResult(String str, int x, int y) {
//*******     Graphics g = getGraphics();
//*******     g.drawString(str,x,y);
 }

/**
 * Gets the options and the text values from the checkboxes and 
 * the textfield components. The values are then displayed 
 * at the bottom of the screen.
 */

 private void getValues() {
     printClear();
     printResult(getOrderName(),40,465); //gets the order's name
     printResult(getOrder(),40,485); // gets the order details
     printResult(getAddress(),40,505); // gets the address and phone numbers
     printResult(getPayBy(),40,525); // gets the pay by options
 }

/**
 * Gets the order's name details and prints it out on the screen
 * using the printResult method
 */

 private String getOrderName() {

     StringBuffer strbuf = new StringBuffer("Mr./Ms.  ");
     strbuf.append((name.getText()).trim() + " for a ");
     return strbuf.toString();
 }

/**
 * Gets the order details and prints it out on the screen
 * using the printResult method
 */

 private String getOrder() {

     StringBuffer strbuf = new StringBuffer();
     Checkbox cb;
    
     cb = pizgrp.getSelectedCheckbox();
     strbuf.append(cb.getLabel() + " Pizza to be delivered at ");
     return strbuf.toString();
 }

/**
 * Gets the address details and prints it out on the screen
 * using the printResult method
 */

 private String getAddress() {
     StringBuffer strbuf = new StringBuffer();

     strbuf.append((st.getText()).trim() + ", ");
     if((city.getText().trim()).length() > 0) 
         strbuf.append((st.getText()).trim() + ", ");

     strbuf.append(" PH # " +(ph.getText()).trim());
     return strbuf.toString();
 }



/**
 * Gets the method of payment and prints it out on the screen
 * using the printResult method
 */

 private String getPayBy() {
     StringBuffer strbuf = new StringBuffer();
     Checkbox cb;

     cb = payby.getSelectedCheckbox();
     strbuf.append("Pay By " + cb.getLabel()) ;
     return strbuf.toString();
 }

} //end of class definition
